"use strict";

var _layouter = _interopRequireDefault(require("../src/layouter.js"));

var _imagesMock = require("./images-mock.js");

var _chai = require("chai");

var _deepFreeze = _interopRequireDefault(require("deep-freeze"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _createForOfIteratorHelperLoose(o) { var i = 0; if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) { if (Array.isArray(o) || (o = _unsupportedIterableToArray(o))) return function () { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }; throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } i = o[Symbol.iterator](); return i.next.bind(i); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(n); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

var getItems = function getItems(count) {
  return (0, _deepFreeze["default"])(_imagesMock.testImages.slice(0, count));
};

var getGroupCount = function getGroupCount(layout) {
  return layout.columns.reduce(function (count, column) {
    return count + column.length;
  }, 0);
};

describe('Layouter', function () {
  var gallery;
  var layouter;
  var container = {};
  var styleParams = {};

  var getLayout = function getLayout(layoutParams) {
    return layouter.createLayout(layoutParams);
  };

  beforeEach(function () {
    var items = getItems();
    styleParams = {
      oneRow: false,
      isVertical: false,
      gallerySize: 200,
      groupSize: 3,
      groupTypes: '1,2h,2v,3t,3b,3l,3r,3v,3h',
      rotatingGroupTypes: '',
      cubeImages: false,
      cubeType: 'fill',
      cubeRatio: 1,
      smartCrop: false,
      chooseBestGroup: true,
      collageAmount: 0.9,
      collageDensity: 0.9,
      minItemSize: 20,
      layoutsVersion: 2,
      galleryMargin: 0,
      imageMargin: 10,
      floatingImages: 0,
      fixedColumns: 0
    };
    container = {
      width: 1000,
      bounds: {
        visibleTop: 0,
        visibleBottom: 1000,
        renderedTop: 0,
        renderedBottom: 10000
      }
    };
    layouter = new _layouter["default"]({
      items: items,
      container: container,
      styleParams: styleParams
    });
  });
  describe('items', function () {
    it('should include all items in original order', function () {
      styleParams.galleryWidth = 500;
      styleParams.minItemSize = 160;

      for (var _i = 0, _arr = [10, 50, 100]; _i < _arr.length; _i++) {
        var size = _arr[_i];
        var items = getItems(size);
        gallery = getLayout({
          items: items,
          container: container,
          styleParams: styleParams
        });
        var galleryItemIds = gallery.items.map(function (item) {
          return item.id;
        });
        var itemIds = items.map(function (item) {
          return item.id || item.photoId;
        });
        (0, _chai.expect)(galleryItemIds).to.deep.equal(itemIds);
      }
    });
    it('should have items offsets and dimensions calculated correctly', function () {
      var items = getItems(100);
      styleParams.galleryWidth = 4000;
      styleParams.gallerySize = 500;
      styleParams.rotatingGroupTypes = '1,2h,2v,3r,3t,3l,3b,3v,3h';
      styleParams.imageMargin = 0;
      gallery = getLayout({
        items: items,
        container: container,
        styleParams: styleParams
      });
      gallery.groups.forEach(function (group) {
        group.items.forEach(function (item) {
          var widthDiff = Math.abs(item.offset.right - item.offset.left - item.width);
          var heightDiff = Math.abs(item.offset.bottom - item.offset.top - item.height);
          (0, _chai.expect)(widthDiff).to.be.below(1);
          (0, _chai.expect)(heightDiff).to.be.below(1);
        });
      });
    });
  });
  describe('container', function () {
    it('should support legacy container props', function () {
      var items = getItems(); //old gallries already include the margin compensation

      var containerA = {
        galleryWidth: 120,
        galleryHeight: 210
      };
      var layouterA = new _layouter["default"]({
        items: items,
        container: containerA,
        styleParams: styleParams
      }); //new galleries don't need to compensate

      var containerB = {
        width: 100,
        height: 200
      };
      var layouterB = new _layouter["default"]({
        items: items,
        container: containerB,
        styleParams: styleParams
      });
      (0, _chai.expect)(layouterA.container).to.deep.equal(layouterB.container);
    });
  });
  describe('Style Params', function () {
    //gallerySize
    it('should have taller Strips as gallerySize increases', function () {
      var items = getItems(100);
      styleParams.isVertical = false;
      var lastGroupHeight = 0;

      for (var _i2 = 0, _arr2 = [100, 200, 300, 400]; _i2 < _arr2.length; _i2++) {
        var size = _arr2[_i2];
        styleParams.gallerySize = size;
        gallery = getLayout({
          items: items,
          container: container,
          styleParams: styleParams
        });
        var maxGroupHeight = gallery.columns[0].groups.reduce(function (g, group) {
          var isLastStrip = group.stripIdx === gallery.strips.length;
          return Math.max(g, isLastStrip ? 0 : group.height);
        }, 0);
        (0, _chai.expect)(maxGroupHeight).to.be.above(lastGroupHeight);
        lastGroupHeight = maxGroupHeight;
      }
    }); //gallerySize

    it('should have all groups in a Columns gallery narrower than gallerySize', function () {
      var items = getItems(100);
      styleParams.isVertical = true;
      styleParams.galleryWidth = 1200;
      var lastGroupWidth = 0;

      for (var _i3 = 0, _arr3 = [10, 50, 100, 200, 300, 400]; _i3 < _arr3.length; _i3++) {
        var size = _arr3[_i3];
        styleParams.gallerySize = size;
        gallery = getLayout({
          items: items,
          container: container,
          styleParams: styleParams
        });
        var maxGroupWidth = gallery.columns.reduce(function (c, column) {
          return c && column.groups.reduce(function (g, group) {
            return Math.max(g, group.width);
          }, true);
        }, true);
        (0, _chai.expect)(maxGroupWidth).to.be.above(lastGroupWidth);
        lastGroupWidth = maxGroupWidth;
      }
    }); //cubeRatio

    it('should have all images in a grid gallery in the required ratio', function () {
      var allowedRounding = 2; //the number of pixels that can change due to rounding

      var items = getItems(100);
      styleParams.cubeImages = true;
      styleParams.imageMargin = 0;

      for (var _i4 = 0, _arr4 = [0.25, 0.5, 1, 2, 4]; _i4 < _arr4.length; _i4++) {
        var ratio = _arr4[_i4];
        styleParams.cubeRatio = ratio;
        gallery = getLayout({
          items: items,
          container: container,
          styleParams: styleParams
        });
        var isCroppedCorrectly = gallery.columns[0].groups.reduce(function (g, group) {
          return g && group.isLastGroup || group.items.reduce(function (i, image) {
            //ignore the last group (it is stretched to fill the galleryWidth)
            var isItemCroppedCorrectly = (image.width - allowedRounding) / (image.height + allowedRounding) <= image.cropRatio && (image.width + allowedRounding) / (image.height - allowedRounding) >= image.cropRatio;
            return i && isItemCroppedCorrectly;
          }, true);
        }, true);
        (0, _chai.expect)(isCroppedCorrectly).to.be["true"];
      }
    }); //fixedColumns

    it('should have fixed number of columns if specified', function () {
      var items = getItems(100);
      styleParams.isVertical = true;

      for (var _i5 = 0, _arr5 = [1, 5, 10, 20]; _i5 < _arr5.length; _i5++) {
        var num = _arr5[_i5];
        styleParams.fixedColumns = num;
        gallery = getLayout({
          items: items,
          container: container,
          styleParams: styleParams
        });
        (0, _chai.expect)(gallery.columns.length).to.equal(num);
      }
    }); //collageAmount

    it('should have more items in groups when the collageAmount increases', function () {
      var itemCount = 100;
      var items = getItems(itemCount);
      var collageAmounts = Array.from({
        length: 11
      }, function (_, i) {
        return i;
      }).map(function (i) {
        return i / 10;
      });
      var lastGroupCount = itemCount;
      styleParams.layoutsVersion = 1;

      for (var _iterator = _createForOfIteratorHelperLoose(collageAmounts), _step; !(_step = _iterator()).done;) {
        var collageAmount = _step.value;
        styleParams.collageAmount = collageAmount;
        gallery = getLayout({
          items: items,
          container: container,
          styleParams: styleParams
        });
        var groupCount = getGroupCount(gallery);
        (0, _chai.expect)(groupCount).not.to.be.above(lastGroupCount);
        lastGroupCount = groupCount;
      }
    }); //collageDensity

    it('should have more items in groups when the collageDensity increases', function () {
      var itemCount = 100;
      var items = getItems(itemCount);
      var collageDensities = Array.from({
        length: 11
      }, function (_, i) {
        return i;
      }).map(function (i) {
        return i / 10;
      });
      var lastGroupCount = itemCount;

      for (var _iterator2 = _createForOfIteratorHelperLoose(collageDensities), _step2; !(_step2 = _iterator2()).done;) {
        var collageDensity = _step2.value;
        styleParams.collageDensity = collageDensity;
        gallery = getLayout({
          items: items,
          container: container,
          styleParams: styleParams
        });
        var groupCount = getGroupCount(gallery);
        (0, _chai.expect)(groupCount).not.to.be.above(lastGroupCount);
        lastGroupCount = groupCount;
      }
    }); //groupSize

    it('should have all groups at maximum groupSize items', function () {
      var items = getItems(100);

      for (var _i6 = 0, _arr6 = [1, 2, 3]; _i6 < _arr6.length; _i6++) {
        var size = _arr6[_i6];
        styleParams.groupSize = size;
        gallery = getLayout({
          items: items,
          container: container,
          styleParams: styleParams
        });
        var isWithinSize = gallery.columns[0].groups.reduce(function (g, group) {
          var inSize = group.items.length <= styleParams.groupSize;
          return g && inSize;
        }, true);
        (0, _chai.expect)(isWithinSize).to.be["true"];
      }
    }); //groupTypes

    it('should have groups only from the optional groups types ', function () {
      var items = getItems(100);
      var groupTypes = ['1', '1,2h,2v', '1,3b,3l,3r', '1,2h,2v,3v,3h', '1,3t,3b', '1,3v,3h', '1,3r,3b,3v,3h', '1,2h,2v,3v,3h,3l,3b']; //groupType '1' must always be an option

      for (var _i7 = 0, _groupTypes = groupTypes; _i7 < _groupTypes.length; _i7++) {
        var type = _groupTypes[_i7];
        styleParams.groupTypes = type;
        gallery = getLayout({
          items: items,
          container: container,
          styleParams: styleParams
        });
        var isWithinTypes = gallery.columns[0].groups.reduce(function (g, group) {
          var inTypes = styleParams.groupTypes.indexOf(group.type) >= 0;
          return g && inTypes;
        }, true);
        (0, _chai.expect)(isWithinTypes).to.be["true"];
      }
    }); //minItemSize

    it('should have all Strips GalleryLayout images larger than minItemSize', function () {
      var items = getItems(100);
      styleParams.isVertical = false;
      styleParams.imageMargin = 0;
      var minItemSizes = [10, 50, 100, 200, 300, 400];

      for (var _i8 = 0, _minItemSizes = minItemSizes; _i8 < _minItemSizes.length; _i8++) {
        var size = _minItemSizes[_i8];
        styleParams.gallerySize = size * 4; //gallerySize must be greater than minItemSize (otherwise the images' proportions will affect the minDimension)

        styleParams.minItemSize = size;
        gallery = getLayout({
          items: items,
          container: container,
          styleParams: styleParams
        });
        var minItemSize = gallery.columns[0].groups.reduce(function (g, group) {
          return group.items.reduce(function (i, item) {
            var minDimension = Math.min(item.width, item.height);
            return Math.min(i, minDimension);
          }, styleParams.minItemSize);
        }, styleParams.minItemSize);
        (0, _chai.expect)(minItemSize).not.to.be.below(size);
      }
    }); //minItemSize

    it('should have all Columns GalleryLayout images larger than minItemSize', function () {
      var items = getItems(100);
      styleParams.isVertical = true;
      styleParams.galleryWidth = 4000;
      styleParams.imageMargin = 0;
      styleParams.collageDensity = 1;
      var minItemSizes = [10, 50, 100, 200, 300];

      for (var _i9 = 0, _minItemSizes2 = minItemSizes; _i9 < _minItemSizes2.length; _i9++) {
        var size = _minItemSizes2[_i9];
        styleParams.gallerySize = size * 8; //gallerySize must be greater than minItemSize (otherwise the images' proportions will affect the minDimension)

        styleParams.minItemSize = size;
        gallery = getLayout({
          items: items,
          container: container,
          styleParams: styleParams
        });
        var minItemSize = gallery.columns.reduce(function (c, column) {
          return c && column.groups.reduce(function (g, group) {
            return Math.min(g, group.items.reduce(function (i, item) {
              var maxDimension = Math.max(item.width, item.height);
              return Math.min(i, maxDimension);
            }, styleParams.minItemSize));
          }, styleParams.minItemSize);
        }, styleParams.minItemSize);
        (0, _chai.expect)(minItemSize).not.to.be.below(size);
      }
    }); // isVertical

    it('should create vertical layouts if isVertical is true', function () {
      var items = getItems(100);
      styleParams.gallerySize = 200;
      styleParams.fixedColumns = 0;
      container.galleryWidth = 1000;
      styleParams.isVertical = true;
      gallery = getLayout({
        items: items,
        container: container,
        styleParams: styleParams
      });
      (0, _chai.expect)(gallery.columns.length).to.equal(5);
      styleParams.isVertical = false;
      gallery = getLayout({
        items: items,
        container: container,
        styleParams: styleParams
      });
      (0, _chai.expect)(gallery.columns.length).to.equal(1);
    }); // oneRow

    it('should create one long row of items if oneRow is true', function () {
      var items = getItems(100);
      container.galleryHeight = 500;
      styleParams.oneRow = false;
      styleParams.imageMargin = 0;
      gallery = getLayout({
        items: items,
        container: container,
        styleParams: styleParams
      });
      (0, _chai.expect)(gallery.height).to.be.above(container.galleryHeight);
      styleParams.oneRow = true;
      gallery = getLayout({
        items: items,
        container: container,
        styleParams: styleParams
      });
      (0, _chai.expect)(gallery.height).to.equal(container.galleryHeight);
    }); // rotatingGroupTypes

    it('should have groups from the rotating groups types by their order ', function () {
      var items = getItems(100);
      styleParams.isVertical = false;
      var groupTypes = ['1', '1,2h,2v', '1,3b,1,3r', '1,2h,2v,3v,3h', '1,3t,3b', '1,3v,3h', '1,3r,2h,3v,3h', '2h,2v,3v,3h,3l,3b'];

      for (var _i10 = 0, _groupTypes2 = groupTypes; _i10 < _groupTypes2.length; _i10++) {
        var type = _groupTypes2[_i10];
        styleParams.rotatingGroupTypes = type;
        gallery = getLayout({
          items: items,
          container: container,
          styleParams: styleParams
        });
        var isWithinTypes = gallery.columns[0].groups.reduce(function (g, group, idx) {
          var rotatingGroupTypes = styleParams.rotatingGroupTypes.split(',');
          var expectedType = rotatingGroupTypes[idx % rotatingGroupTypes.length];
          var groupType = group.type;
          (0, _chai.expect)(expectedType).to.equal(groupType);
          var isType = expectedType === groupType;
          return g && isType;
        }, true);
        (0, _chai.expect)(isWithinTypes).to.be["true"];
      }
    }); // cubeImages

    it('should have all images in their original ratio if cubeImages is false', function () {
      var allowedRounding = 2; //the number of pixels that can change due to rounding

      var items = getItems(100); //todo - something breaks when using exactly 100 images

      styleParams.cubeImages = false;
      styleParams.imageMargin = 0;
      styleParams.collageDensity = 0.8;
      gallery = getLayout({
        items: items,
        container: container,
        styleParams: styleParams
      });
      var isOriginalDimensions = gallery.columns[0].groups.reduce(function (g, group) {
        return g && group.isLastGroup || group.items.reduce(function (i, image) {
          //ignore the last group (it is stretched to fill the galleryWidth)
          var isItemCroppedCorrectly = (image.width - allowedRounding) / (image.height + allowedRounding) <= image.maxWidth / image.maxHeight && (image.width + allowedRounding) / (image.height - allowedRounding) >= image.maxWidth / image.maxHeight;
          return i && isItemCroppedCorrectly; //ignore fractions
        }, true);
      }, true);
      (0, _chai.expect)(isOriginalDimensions).to.be["true"];
      styleParams.cubeImages = true;
      gallery = getLayout({
        items: items,
        container: container,
        styleParams: styleParams
      });
      var isCroppedCorrectly = gallery.columns[0].groups.reduce(function (g, group) {
        return g && group.items.reduce(function (i, image) {
          var isItemCroppedCorrectly = (image.width - allowedRounding) / (image.height + allowedRounding) <= styleParams.cubeRatio && (image.width + allowedRounding) / (image.height - allowedRounding) >= styleParams.cubeRatio;
          return i && isItemCroppedCorrectly;
        }, true);
      }, true);
      (0, _chai.expect)(isCroppedCorrectly).to.be["true"];
    }); // smartCrop

    it('should crop images according to their original proportions if smartCrop is true', function () {
      var allowedRounding = 2; //the number of pixels that can change due to rounding

      var items = getItems(100);
      styleParams.cubeRatio = 2;
      styleParams.cubeImages = true;
      styleParams.smartCrop = true;
      styleParams.imageMargin = 0;
      gallery = getLayout({
        items: items,
        container: container,
        styleParams: styleParams
      });
      var isCroppedCorrectly = gallery.columns[0].groups.reduce(function (g, group) {
        return g && group.items.reduce(function (i, image) {
          var cropRatio = image.isLandscape ? styleParams.cubeRatio : 1 / styleParams.cubeRatio;
          var isItemCroppedCorrectly = (image.width - allowedRounding) / (image.height + allowedRounding) <= cropRatio && (image.width + allowedRounding) / (image.height - allowedRounding) >= cropRatio;
          return i && isItemCroppedCorrectly;
        }, true);
      }, true);
      (0, _chai.expect)(isCroppedCorrectly).to.be["true"];
    }); // chooseBestGroup

    it('should not allow ugly groups if chooseBestGroup is true ', function () {
      var items = getItems(99);
      styleParams.groupTypes = '3t,3r,3l,3b'; //without 1

      styleParams.groupSize = 3;
      styleParams.collageDensity = 1;
      styleParams.minItemSize = 10;
      styleParams.gallerySize = 1000;

      for (var _i11 = 0, _arr7 = [true, false]; _i11 < _arr7.length; _i11++) {
        var chooseBestGroup = _arr7[_i11];
        styleParams.chooseBestGroup = chooseBestGroup;
        gallery = getLayout({
          items: items,
          container: container,
          styleParams: styleParams
        });
        var isWithinTypes = gallery.columns[0].groups.reduce(function (g, group) {
          var groupType = group.type;
          var isType = groupType !== '1'; //if ChoooseBestGroup is true, some group will have to be single

          return g && isType;
        }, true);
        (0, _chai.expect)(isWithinTypes).to.not.equal(chooseBestGroup);
      }
    }); // imageMargin (between groups)

    it('should have spaces between groups equal to imageMargin', function () {
      var items = getItems(100);
      styleParams.galleryWidth = 4000;
      styleParams.gallerySize = 500;
      styleParams.groupSize = 1;

      var _loop = function _loop() {
        var margin = _arr8[_i12];
        styleParams.imageMargin = margin;
        gallery = getLayout({
          items: items,
          container: container,
          styleParams: styleParams
        });
        var lastItem = false;
        var marginDiff = 0;
        var totalMarginDiff = gallery.items.reduce(function (i, item) {
          if (lastItem) {
            var realMargin;

            if (lastItem.offset.top === item.offset.top) {
              realMargin = Math.round(item.offset.left - lastItem.offset.right);
            } else {
              realMargin = Math.round(item.offset.top - lastItem.offset.bottom);
            }

            marginDiff = Math.abs(realMargin - margin * 2);
            (0, _chai.expect)(marginDiff).to.be.below(1);
          }

          lastItem = item;
          return i + Math.floor(marginDiff);
        }, 0);
        (0, _chai.expect)(totalMarginDiff).to.be.below(1);
      };

      for (var _i12 = 0, _arr8 = [10, 50, 100, 200]; _i12 < _arr8.length; _i12++) {
        _loop();
      }
    }); // imageMargin (within groups)
    //TODO fix this test once the playground is complete

    it('should have spaces between items in a group equal to imageMargin', function () {
      var items = getItems(100);
      styleParams.galleryWidth = 4000;
      styleParams.gallerySize = 1000;
      styleParams.groupSize = 3;
      styleParams.groupTypes = '1,2h,2v,3r,3t,3l,3b,3v,3h';

      var _loop2 = function _loop2() {
        var margin = _arr9[_i13];
        styleParams.imageMargin = margin;
        gallery = getLayout({
          items: items,
          container: container,
          styleParams: styleParams
        });
        var marginDiff = 0;
        gallery.columns[0].groups.reduce(function (g, group) {
          var lastItem = false;
          var groupMarginDiff = group.items.reduce(function (i, item) {
            if (lastItem) {
              var realMargin;

              if (lastItem.offset.top < item.offset.top) {
                realMargin = Math.round(item.offset.top - lastItem.offset.bottom);
              } else {
                realMargin = Math.round(item.offset.left - lastItem.offset.right);
              }

              marginDiff = Math.abs(realMargin - margin * 2);
              (0, _chai.expect)(marginDiff).to.be.below(2);
            }

            lastItem = item;
            return i + Math.floor(marginDiff);
          }, 0);
          return g + groupMarginDiff;
        }, 0);
      };

      for (var _i13 = 0, _arr9 = [0, 30, 40, 80]; _i13 < _arr9.length; _i13++) {
        _loop2();
      }
    }); // rotatingGroupTypes

    it('should type groups according to rotatingGroupTypes if defined', function () {
      var items = getItems(100); //todo - something breaks when using exactly 100 images

      styleParams.rotatingGroupTypes = '2h,3v,3b,3t,1,2h,2v';
      var rotatingGroupTypesArr = styleParams.rotatingGroupTypes.split(',');
      gallery = getLayout({
        items: items,
        container: container,
        styleParams: styleParams
      });
      gallery.groups.forEach(function (group, g) {
        (0, _chai.expect)(group.type).to.equal(rotatingGroupTypesArr[g % rotatingGroupTypesArr.length]); //first group idx is 1
      }, true);
    }); // functional cropRatio

    it('should crop items according to the cropRatio function if defined', function () {
      var items = getItems(100); //todo - something breaks when using exactly 100 images

      styleParams.cropRatio = function () {
        return Math.random();
      };

      styleParams.cropItems = true;
      styleParams.smartCrop = false;
      gallery = getLayout({
        items: items,
        container: container,
        styleParams: styleParams
      });
      gallery.items.reduce(function (i, item) {
        (0, _chai.expect)(item.cropRatio).to.not.equal(i);
        return item.cropRatio;
      }, 0);
    }); // crop only fill

    it('should not crop items if cropOnlyFill is true and cropType is fit', function () {
      var items = getItems(100); //todo - something breaks when using exactly 100 images

      styleParams.cropRatio = 1;
      styleParams.cropOnlyFill = true;
      styleParams.cubeType = 'fit';
      styleParams.cropItems = true;
      styleParams.smartCrop = false;
      gallery = getLayout({
        items: items,
        container: container,
        styleParams: styleParams
      });
      gallery.items.forEach(function (item) {
        (0, _chai.expect)(item.cropRatio).to.equal(item.ratio);
      });
    }); // rotatingCropRatios

    it('should crop items according to rotatingCropRatios if defined', function () {
      var items = getItems(100); //todo - something breaks when using exactly 100 images

      styleParams.rotatingCropRatios = '0.2,0.6,8,4.5';
      styleParams.cropRatio = '1';
      styleParams.cropItems = true;
      styleParams.smartCrop = false;
      var rotatingCropRatiosArr = styleParams.rotatingCropRatios.split(',');
      gallery = getLayout({
        items: items,
        container: container,
        styleParams: styleParams
      });
      gallery.items.forEach(function (item, i) {
        (0, _chai.expect)(item.cropRatio).to.equal(Number(rotatingCropRatiosArr[i % rotatingCropRatiosArr.length]));
      }, true);
    });
  });
  describe('Infinite Scroll', function () {
    it('scroll should increase visible items range', function () {
      var items = getItems(100);
      styleParams.isVertical = true;
      styleParams.fixedColumns = 2;
      styleParams.groupSize = 1;
      container.galleryWidth = 1000;
      container.galleryHeight = 1000;

      var isVisible = function isVisible(itemIdx) {
        var item = gallery.items[itemIdx];
        return !!item && item.visibility.visible;
      };

      var visibleItemsRange = function visibleItemsRange(_items) {
        var from = false;
        var to = false;

        for (var _iterator3 = _createForOfIteratorHelperLoose(_items), _step3; !(_step3 = _iterator3()).done;) {
          var item = _step3.value;

          if (isVisible(item.idx)) {
            if (from === false) {
              from = item.idx;
            }
          } else if (to === false && from !== false) {
            to = item.idx - 1;
          }
        }

        return [from, to];
      };

      var scroll = function scroll(base) {
        return {
          renderedTop: base - 3000,
          visibleTop: base,
          visibleBottom: base + 1000,
          renderedBottom: base + 4000
        };
      };

      var lastFrom = -1;
      var lastTo = -1;
      var boundsOptions = [0, 1000, 2500, 6000].map(function (base) {
        return scroll(base);
      });
      gallery = layouter.createLayout({
        items: items,
        container: container,
        styleParams: styleParams
      });

      for (var _iterator4 = _createForOfIteratorHelperLoose(boundsOptions), _step4; !(_step4 = _iterator4()).done;) {
        var bounds = _step4.value;
        gallery = layouter.calcVisibilities(bounds);

        var _visibleItemsRange = visibleItemsRange(gallery.items),
            from = _visibleItemsRange[0],
            to = _visibleItemsRange[1];

        (0, _chai.expect)(isVisible(from - 1)).to.be["false"];
        (0, _chai.expect)(isVisible(from)).to.be["true"];
        (0, _chai.expect)(isVisible(to)).to.be["true"];
        (0, _chai.expect)(isVisible(to + 1)).to.be["false"];
        (0, _chai.expect)(lastFrom).to.be.below(from);
        (0, _chai.expect)(lastTo).to.be.below(to);
        lastFrom = from;
        lastTo = to;
      }
    });
  });
  describe('public methods', function () {
    it('findLastVisibleItemIdx should work', function () {
      var items = getItems(100);
      styleParams.isVertical = true;
      styleParams.fixedColumns = 1;
      styleParams.groupSize = 1;
      container.galleryWidth = 1000;
      var galleryHeights = [500, 1000, 2500, 6000];

      for (var _i14 = 0, _galleryHeights = galleryHeights; _i14 < _galleryHeights.length; _i14++) {
        var galleryHeight = _galleryHeights[_i14];
        container.galleryHeight = galleryHeight;
        gallery = layouter.createLayout({
          items: items,
          container: container,
          styleParams: styleParams
        });
        var lastVisibleItemIdx = layouter.lastVisibleItemIdx();
        var lastVisibleItem = gallery.items[lastVisibleItemIdx];
        var firstHiddenItem = gallery.items[lastVisibleItemIdx + 1];
        (0, _chai.expect)(lastVisibleItem.offset.top).to.be.below(galleryHeight);
        (0, _chai.expect)(firstHiddenItem.offset.top).to.be.above(galleryHeight);
      }
    });
    it('findNeighborItem should work', function () {
      var neighbor;
      var items = getItems(100);
      styleParams.isVertical = true;
      styleParams.cubeImages = true;
      styleParams.cubeRatio = 1;
      styleParams.groupSize = 1;
      container.galleryWidth = 1000;

      for (var _i15 = 0, _arr10 = [2, 3, 5, 7]; _i15 < _arr10.length; _i15++) {
        var fixedColumns = _arr10[_i15];
        styleParams.fixedColumns = fixedColumns;
        gallery = layouter.createLayout({
          items: items,
          container: container,
          styleParams: styleParams
        });

        for (var _iterator5 = _createForOfIteratorHelperLoose(gallery.items), _step5; !(_step5 = _iterator5()).done;) {
          var item = _step5.value;

          if (item.idx > fixedColumns) {
            neighbor = layouter.findNeighborItem(item.idx, 'up');
            (0, _chai.expect)(item.idx - neighbor).to.equal(fixedColumns);
          }

          if (item.idx < gallery.items.length - fixedColumns - 1) {
            neighbor = layouter.findNeighborItem(item.idx, 'down');
            (0, _chai.expect)(neighbor - item.idx).to.equal(fixedColumns);
          }

          if (item.idx % fixedColumns > 0) {
            neighbor = layouter.findNeighborItem(item.idx, 'left');
            (0, _chai.expect)(item.idx - neighbor).to.equal(1);
          }

          if (item.idx % fixedColumns < fixedColumns - 1 && item.idx < gallery.items.length - 1) {
            neighbor = layouter.findNeighborItem(item.idx, 'right');
            (0, _chai.expect)(neighbor - item.idx).to.equal(1);
          }
        }
      }
    });
  });
});